/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlBinop.h"
#include "HvSignalEnvelope.h"
#include "HvSignalVar.h"
#include "HvControlCast.h"
#include "HvSignalLine.h"
#include "HvMath.h"
#include "HvControlIf.h"
#include "HvControlVar.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_M6asNL0K_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_pD8efxEX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YcIeEKir_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_oJL2G03A_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_rLH82p8e_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_K1tvsicg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_E8UKIJyU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_SBMq28EM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_yOvXRzK7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4I4BCYnJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5qw0NOV2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Uc138ZJz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_K0kQz6mZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_iIbIWO2A_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_DjTVw53o_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_VkV4sjZC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_bcJdxwBh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_bjRrU1Rt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_WcDB3teZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_839gXfWd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Uw1eh8k5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_C01A3kQj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BZNUiiv4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_WeuZiHCR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_GAIN9dfK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dhdsSivL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_8epRfq6M_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_obbMIWO9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_SZK89bkM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_vxXGj6MO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_13HnNapu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_9lYyx0VL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_S32KMjmt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_u2fX3up1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_tAcwWkds_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_1buTlMfD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_7UYTlTfq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_KrzncZCw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_S36PsfWj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RiVjJw5M_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qxDE3yPz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_8bd9qeac_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dxNDywzs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_vyyjekDR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_6U9VVBMX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Q1iHbxR5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_dxMKQTFH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_0PUe4Joa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_uWK6k40b_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_nz0li8lD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_MjyClvS3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_zqo9gLf3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_YWVHPDHs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_J4LKiMos_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_UrdnpQEQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_K3Qq4QVv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_klURwebk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Dt2EPMd0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_l8o0yH3H_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_FwUNW9CV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_tnaeRcoR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_5wJ5H3sY_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_pcXUFGpw;
  SignalEnvelope sEnv_bjRrU1Rt;
  SignalEnvelope sEnv_WcDB3teZ;
  SignalEnvelope sEnv_tAcwWkds;
  SignalEnvelope sEnv_1buTlMfD;
  SignalLine sLine_66lOiY8t;
  SignalLine sLine_3Z0YWsak;
  ControlBinop cBinop_M6asNL0K;
  ControlBinop cBinop_pD8efxEX;
  ControlBinop cBinop_YcIeEKir;
  ControlBinop cBinop_oJL2G03A;
  ControlBinop cBinop_E8UKIJyU;
  ControlBinop cBinop_yOvXRzK7;
  ControlBinop cBinop_4I4BCYnJ;
  ControlBinop cBinop_5qw0NOV2;
  ControlVar cVar_Uc138ZJz;
  ControlVar cVar_iIbIWO2A;
  SignalVarf sVarf_3ImN7C2p;
  ControlVar cVar_DjTVw53o;
  ControlIf cIf_VkV4sjZC;
  ControlIf cIf_bcJdxwBh;
  ControlBinop cBinop_839gXfWd;
  ControlBinop cBinop_C01A3kQj;
  ControlBinop cBinop_BZNUiiv4;
  ControlBinop cBinop_dhdsSivL;
  ControlBinop cBinop_obbMIWO9;
  SignalVarf sVarf_Ig0ooEBJ;
  ControlVar cVar_9lYyx0VL;
  ControlIf cIf_S32KMjmt;
  ControlIf cIf_u2fX3up1;
  ControlBinop cBinop_7UYTlTfq;
  ControlBinop cBinop_S36PsfWj;
  ControlBinop cBinop_RiVjJw5M;
  ControlBinop cBinop_dxNDywzs;
  ControlBinop cBinop_6U9VVBMX;
  ControlVar cVar_uWK6k40b;
  ControlBinop cBinop_nz0li8lD;
  SignalVarf sVarf_qasAIYny;
  SignalVarf sVarf_k3btL4v5;
  SignalVarf sVarf_Z0LTlc2V;
  SignalVarf sVarf_eOIx5ilT;
  ControlBinop cBinop_J4LKiMos;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
